package com.conflux.web.controller.util;

import com.alibaba.fastjson.JSONObject;
import okhttp3.*;
import org.apache.commons.lang3.StringUtils;

import javax.net.ssl.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetSocketAddress;
import java.net.Proxy;
import java.security.cert.X509Certificate;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * 网路访问
 *
 * @author wangban
 * @date 11:59 2018/9/10
 */
public class OkHttpsUtils {

    private static OkHttpClient mOkHttpClient;

    /**
     * 网络访问连接池
     */
    private static ConnectionPool pool;

    /**
     * get 请求
     *
     * @param url 请求地址
     * @return 返回结果
     */
    public static String getRequest(String url, String appKey) {
        try {

            Request.Builder builder = new Request.Builder();
            if (!StringUtils.isEmpty(appKey)) {
                builder.addHeader("Authorization", "APPCODE" + appKey);
            }
            builder.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36");
            builder.url(url);
            final Request request = builder.build();
            //new call
            Call call = getOkHttpInstance().newCall(request);
            //请求加入调度
            Response response = call.execute();
            return response.body().string();
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * get 请求 InputStream
     *
     * @param url 请求地址
     * @return 返回结果
     */
    public static InputStream getInputStreamRequest(String url) {
        try {

            Request.Builder builder = new Request.Builder();
            builder.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36");
            builder.url(url);
            final Request request = builder.build();
            //new call
            Call call = getOkHttpInstance().newCall(request);
            //请求加入调度
            Response response = call.execute();
            return response.body().byteStream();
        } catch (IOException e) {
            System.err.println(e.getMessage());
            return null;
        }
    }

    private static OkHttpClient getOkHttpInstance() {
        if (mOkHttpClient == null) {
            synchronized (OkHttpsUtils.class) {
                //创建okHttpClient对象
                mOkHttpClient = getUnsafeOkHttpClient();
            }
        }
        return mOkHttpClient;
    }


    //okHttp3添加信任所有证书

    public static OkHttpClient getUnsafeOkHttpClient() {

        X509TrustManager x509 = new X509TrustManager() {
            @Override
            public void checkClientTrusted(X509Certificate[] chain, String authType) {
            }

            @Override
            public void checkServerTrusted(X509Certificate[] chain, String authType) {
            }

            @Override
            public X509Certificate[] getAcceptedIssuers() {
                return new X509Certificate[]{};
            }
        };

        try {
            final TrustManager[] trustAllCerts = new TrustManager[]{x509};

            final SSLContext sslContext = SSLContext.getInstance("SSL");
            sslContext.init(null, trustAllCerts, new java.security.SecureRandom());
            final SSLSocketFactory sslSocketFactory = sslContext.getSocketFactory();
            OkHttpClient.Builder builder = new OkHttpClient.Builder()
                    .connectTimeout(3, TimeUnit.MINUTES)
                    .writeTimeout(3, TimeUnit.MINUTES)
                    .readTimeout(3, TimeUnit.MINUTES);
            builder.sslSocketFactory(sslSocketFactory, x509);

            builder.hostnameVerifier(new HostnameVerifier() {
                @Override
                public boolean verify(String hostname, SSLSession session) {
                    return true;
                }
            });

            return builder.build();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    /**
     * 获取网络访问连接池
     *
     * @return
     */
    private static ConnectionPool getConnectionPool() {
        if (pool == null) {
            synchronized (OkHttpsUtils.class) {
                pool = new ConnectionPool(5, 10, TimeUnit.MINUTES);
            }
        }
        return pool;
    }


    /**
     * get 请求 含代理
     *
     * @param url 请求地址
     * @return 返回结果
     */
    public static String getRequestByProxy(String url, String host, int port) {
        try {
            //创建okHttpClient对象
            OkHttpClient.Builder b = new OkHttpClient.Builder();
            Proxy proxy = new Proxy(Proxy.Type.HTTP, new InetSocketAddress(host, port));
            b.proxy(proxy);
            //创建一个Request
            Request.Builder builder = new Request.Builder();
            builder.addHeader("User-Agent", "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36");
            builder.url(url);
            final Request request = builder.build();
            //new call
            Call call = getOkHttpInstance().newCall(request);
            //请求加入调度
            Response response = call.execute();
            return response.body().string();
        } catch (IOException e) {
            System.err.println(e.getMessage());
            return "";
        }
    }


    public static String uploadObject(String url, File file, String token) throws IOException {
        MediaType mediaType = MediaType.parse("image/*");
        RequestBody requestBody = RequestBody.create(mediaType, file);

        Request request = new Request.Builder()
                .url(url)
                .post(requestBody)
                .addHeader("Authorization", token)
                .build();

        Call call = getOkHttpInstance().newCall(request);
        Response response = call.execute();
        return response.body().string();
    }

    /**
     * 一般post请求
     *
     * @param url    请求路径
     * @param params 请求参数
     * @param token  消息头
     * @return 返回结果
     */
    public static String postRequest(String url, Map<String, String> params, String token) {
        try {

            FormBody.Builder formBodyBuilder = new FormBody.Builder();
            Set<String> keySet = params.keySet();
            for (String key : keySet) {
                String value = params.get(key);
                formBodyBuilder.add(key, value);
            }
            FormBody formBody = formBodyBuilder.build();

            Request.Builder builder = new Request.Builder();
            if (!StringUtils.isEmpty(token)) {
                builder.addHeader("Authorization", token);
            }
            Request request = builder.url(url)
                    .post(formBody).build();

            Call call = getOkHttpInstance().newCall(request);

            Response response = call.execute();
            return response.body().string();
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 一般post请求
     *
     * @param url    请求路径
     * @param params 请求参数
     * @param token  消息头
     * @return 返回结果
     */
    public static JSONObject postRequestObj(String url, Map<String, String> params, String token) {
        try {

            FormBody.Builder formBodyBuilder = new FormBody.Builder();
            Set<String> keySet = params.keySet();
            for (String key : keySet) {
                String value = params.get(key);
                formBodyBuilder.add(key, value);
            }
            FormBody formBody = formBodyBuilder.build();

            Request.Builder builder = new Request.Builder();
            if (!StringUtils.isEmpty(token)) {
                builder.addHeader("Authorization", token);
            }
            Request request = builder.url(url)
                    .post(formBody).build();

            Call call = getOkHttpInstance().newCall(request);

            Response response = call.execute();
            String s = response.body().string();

            return JSONObject.parseObject(s);
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * post传json参数
     *
     * @param url  请求路径
     * @param json 请求的json数据
     * @return 返回结果
     */
    public static String postJsonAuth(String url, String json, String token) {
        try {

            //MediaType  设置Content-Type 标头中包含的媒体类型值
            RequestBody requestBody = FormBody.create(MediaType.parse("application/json; charset=utf-8")
                    , json);

            Request.Builder builder = new Request.Builder();
            if (!StringUtils.isEmpty(token)) {
                builder.addHeader("Authorization", token);
            }
            Request request = builder
                    //请求的url
                    .url(url)
                    .post(requestBody)
                    .build();

            //创建/Call
            Call call = getOkHttpInstance().newCall(request);
            Response response = call.execute();
            return response.body().string();
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }


    /**
     * post传json参数
     *
     * @param url  请求路径
     * @param json 请求的json数据
     * @return 返回结果
     */
    public static String postJsonResponse(String url, String json) {
        try {

            //MediaType  设置Content-Type 标头中包含的媒体类型值
            RequestBody requestBody = FormBody.create(MediaType.parse("application/json; charset=utf-8")
                    , json);

            Request request = new Request.Builder()
                    //请求的url
                    .url(url)
                    .post(requestBody)
                    .build();

            //创建/Call
            Call call = getOkHttpInstance().newCall(request);
            Response response = call.execute();
            return response.body().string();
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    /**
     * 一般post请求
     *
     * @param url    请求路径
     * @param params 请求参数
     * @return 返回结果
     */
    public static String postRequest(String url, Map<String, String> params) {
        try {

            FormBody.Builder formBodyBuilder = new FormBody.Builder();
            Set<String> keySet = params.keySet();
            Request.Builder builder = new Request.Builder();
            for (String key : keySet) {
                String value = params.get(key);
                if (!StringUtils.isEmpty(value)) {
                    builder.addHeader(key, value);
                }
            }
            FormBody formBody = formBodyBuilder.build();

            Request request = builder.url(url)
                    .post(formBody).build();

            Call call = getOkHttpInstance().newCall(request);

            Response response = call.execute();
            String s = response.body().string();

            return s;
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }

}

